#include "promdaq.h"

//globals  must be global to the entire project


short CPrm_DAQ::Init(	USHORT baseaddr, 
				UCHAR irqno, 
				UCHAR dmalev, 
				UCHAR cksel1, 
				UCHAR ckfrq1, 
				UCHAR ckfrq0	)
{
	short err; 
	UCHAR input, CounterBits;
	
	
	// If baseaddr not in range 0x10 to 0x3F0 then error
	if(baseaddr < 0x10 || baseaddr > 0x3F0)
	{
		err = 1;
		return(err);	
	}
	
	// If irqno not in range 5 or 11 then error (check schematic)
	if(irqno < 5 || irqno > 11)
	{
		err = 2;
		return(err);
	}
	
	//	If  dmalev not in range 1 or 3 then error (check schematic)
	if(dmalev < 1 || dmalev > 3)
	{
		err = 3;
		return(err);
	}
	
	//	If cksel1 not in range 0-1 then error
	if(cksel1 < 0 || cksel1 > 1)
	{
		err = 4;
		return(err);
	}
	
	// If ckfrq1 not in range 0-1 then error
	if(ckfrq1 < 0 || ckfrq1 > 1)
	{
		err = 5;
		return(err);
	}
	
	//	If ckfrq0 not in range 0-1 then error
	if(ckfrq0 < 0 || ckfrq0 > 1)
	{
		err = 6;
		return(err);
	}
	
	//	Store inputs for later use
	BaseAddress = baseaddr;
	IrqNumber = irqno;
	DmaLevel = dmalev;
	Select1 = cksel1;
	Frequency1 = ckfrq1;
	Frequency0 = ckfrq0;
	
	//	Reset Board to clear all registers
	RESET;
	
	//
	//	Check if board is present by writing / reading channel 
	// and FIFO depth registers
	//
	_outp(baseaddr + 2, 0x55);
	input = _inp(baseaddr + 2);
	if(input != 0x55)
	{
		err = 7;
		return(err);
	}
	
	_outp(baseaddr + 2, 0xAA);
	input = _inp(baseaddr + 2);
	if(input != 0xAA)
	{
		err = 8;
		return(err);
	}
	
	_outp(baseaddr + 2, 0);	//Channel register
	_outp(baseaddr + 5, 0);	//FIFO Threshold
	input = _inp(baseaddr + 5);
	if(input)
	{
		err = 9;
		return(err);
	}
	
	_outp(baseaddr + 5, 0xFF);
	input = _inp(baseaddr + 5);
	if(input != 0x1F)
	{
		err = 0xA;
		return(err);
	}
	
	_outp(baseaddr + 5, 0);
	
	// Reset Board again to clear all registers	
	RESET;
	
	//	Setup counter bits
	CounterBits = (cksel1 << 7) + (ckfrq1 << 6) + (ckfrq0 << 5);
	_outp(baseaddr +4 , CounterBits);
	
	// Successful return
	return(0);
}					




void CPrm_DAQ::ADSetSettings(	UCHAR LowChannel, 
					UCHAR HighChannel, 
					UCHAR Gain	)
{
	UCHAR Value;
	
	//	Set channel range
	Value = ( (HighChannel & 0xF) << 4 ) | (LowChannel & 0xF);
	_outp(BaseAddress + 2, Value);
	
	//	Set gain
	// strip off existing gain bits
	Value = _inp(BaseAddress + 3) & 0xFC;
	_outp( BaseAddress + 3, (Value | (Gain & 3)) );
	
	//	Wait for settling time (WAIT bit)
	do
	{
		Value = _inp(BaseAddress + 3) & 0x20;
	} while (Value);
}								
							

short CPrm_DAQ::ADSample(void)
{
	short ADValue;
	
	ADValue = Do_adsample();
	return(ADValue);
}	


short CPrm_DAQ::Do_adsample(void)
{
	short ADValue;
	short Lsb, Msb;
	
	//	Start conversion
	_outp(BaseAddress, 0x80);
	
//	Wait for finish
	do
	{
		Lsb = _inp(BaseAddress + 3);
	} while (Lsb & 0x80);
	
	//	Read A/D Value
	Lsb = _inp(BaseAddress);
	Msb = _inp(BaseAddress + 1);
	
	ADValue = (Msb << 8) + Lsb;
	return(ADValue);
}	


/*USHORT CPrm_DAQ::ADSingle(UCHAR LowChannel, UCHAR HighChannel, UCHAR Gain)
{
	USHORT ADValue;
	
	ADSetSettings(LowChannel, HighChannel, Gain);
	ADValue = ADSample();
	return(ADValue);	
}*/	


void CPrm_DAQ::ADScan(short *lpArray, short high, short low)
{
	UCHAR input,original;
	USHORT Lsb, Msb;
	
	//	Reset FIFO
	_outp(BaseAddress, 0x10);
	
	//	Enable scan mode
	original = _inp(BaseAddress + 3);
	input=original;
	input |= 4;
	_outp(BaseAddress + 3, input);
	
	// Perform A/D conversions on all channels in list
	//	For (I = 1 to high - low + 1) { Array(i) = do_adsample() } (nonscan method)
	
	//	Start conversions
	_outp(BaseAddress, 0x80);
	
	//	Wait for entire scan to finish
	do
	{
		input = _inp(BaseAddress + 3);
	} while (input & 0x80);
	
	for(int i = 0; i < high - low + 1; i++)
	{
		Lsb = _inp(BaseAddress);
		Msb = _inp(BaseAddress + 1);
		lpArray[i] = (Msb << 8) + Lsb; 
	}
	_outp(BaseAddress + 3, original);	//restore original value
}	



void CPrm_DAQ::ADSampleInt(short FifoThreshold)
{
	DoADInt(FifoThreshold, 0);
}	


void CPrm_DAQ::ADScanInt(short FifoThreshold)
{
	DoADInt(FifoThreshold, 1);
}	


short CPrm_DAQ::DoADInt(short Threshold, short ScanEnable)
{
	USHORT input, err;

	// If another interrupt operation is in progress then return (error)
	if(Int)
	{
		err = 1;
		return(err);
	}
	
	//	If fifothreshold > fifolimit (currently 16) then return (error)
	if(Threshold > 16)
	{
		err = 2;
		return(err);
	}
	
	FifoThreshold = Threshold;
	
	// get current config value; strip off int and dma bits
	input = (BaseAddress + 4) & 0xF0; 	
	
	// stop interrupts temporarily
	_outp (BaseAddress + 4, input);
	
	//	install pointer to interrupt function *ADSampleIntFunction()
	// enable hardware interrupts on board's interrupt level (language/OS dependent)
	
	
	_outp (BaseAddress + 5, FifoThreshold);
	input = _inp(BaseAddress + 3);

	// set SCANEN bit if scanint is caller
	if(ScanEnable == 1)
	{
		_outp(BaseAddress + 3, input | 4);	
	}
	
	// get current config value; strip off int and dma bits 
	// (unneeded repeat of above)
	input = _inp(BaseAddress + 4) & 0xF0;
	// set analog interrupt enable bit
	_outp(BaseAddress + 4, input | 1);	

	Int = 0;
	return(0);
}	



short CPrm_DAQ::DAConvert( USHORT Channel, USHORT DAValue)
{
	short err;
	UCHAR Lsb, Msb;
	 
	if(Channel < 0 || Channel > 3)
	{
		err = 1;
		return(err);
	}
	
	if (DAValue < 0 || DAValue > 4095)
	{
		err = 2;
		return(err);
	}
	
	//	Build LSB, MSB
	Lsb = (UCHAR)DAValue & 0x00FF;
	// truncate values for safety
	Msb =  (UCHAR)( ( (DAValue & 0x0F00) >> 8 ) + ((Channel & 0x03) << 6) );
	
	//	Write data to board
	_outp(BaseAddress + 6, Lsb);
	_outp(BaseAddress + 7, Msb);
	
	//	Wait for finish
	do
	{
		err = _inp(BaseAddress + 3);
	} while (err & 0x10);
	
	//	successful return
	return(0);
}	


void CPrm_DAQ::DAConvertScan(USHORT *lpChannel, USHORT *lpDAValue)
{
	for(int i = 0; i < 4; i++)
	{
		DAConvert(lpChannel[i], lpDAValue[i]);
	}
}	



void CPrm_DAQ::DIOConfig(UCHAR DirA, UCHAR DirB, UCHAR DirCh, UCHAR DirCl, UCHAR CtrIO)
{
	UCHAR cfg;
	
	//	Build config value	
	// board will ignore DIRCH if CTRIO = 1
	cfg = (CtrIO << 7) + (DirA << 4) + (DirCh << 3) + (DirB << 1) + DirCl;
	
	//	Write config value
	_outp(BaseAddress + 11, cfg);
}	



void CPrm_DAQ::DIOConfigDirect(UCHAR config)
{
	//	Write config value
	_outp(BaseAddress + 11, config);
}	


short CPrm_DAQ::DigitalOutByte(USHORT port, UCHAR data)
{
	if(port < 0 || port > 2)
	{
		return(1);
	}
	_outp(BaseAddress + 8 + port, data);
	return(0);
}	




UCHAR CPrm_DAQ::DigitalInByte(USHORT port)
{
	UCHAR input;
	
	if(port < 0 || port > 2)
	{
		return(1);
	}
	
	input = _inp(BaseAddress + 8 + port);
	return(input);
}	


short CPrm_DAQ::DigitalOutBit(USHORT port, USHORT Bitno, short BitVal)
{
	short err;
	USHORT input;
	
	if(port < 0 || port > 2)
	{
		err = 1;
		return(err);
	}
	
	if(Bitno < 0 || Bitno > 7)
	{
		err = 2;
		return(err);
	}
	
	if(BitVal < 0 || BitVal > 1)
	{
		err = 3;
		return(err);
	}
	
	BitVal &= 1;
	
	//	Read port value, update bit, _output new value
	input = _inp(BaseAddress + 8 + port);
	if(!BitVal)
	{
		input &= ~(1 << Bitno);	//clear bit
	}
	else
	{
		input |= (1 << Bitno);	//set bit
	}
	_outp(BaseAddress + 8 + port, input);	//write new value
	
	return(0);
}	


USHORT CPrm_DAQ::DigitalInBit(USHORT port, USHORT Bitno)
{
	USHORT input;
	
	if(port < 0 || port > 2)
	{
		// error
		return(-1);
	}
	
	if(Bitno < 0 || Bitno > 7)
	{
		// error
		return(-1);
	}
	
	//	Read port value and return individual bit value
	input = _inp(BaseAddress + 8 + port);
	input &= 2 ^ Bitno;
	
	input ? input = 1 : input = 0;
	
	return(input);
}


void CPrm_DAQ::CtrConfig(short ctrno, UCHAR clkfrq, UCHAR clksel)
{
	USHORT input;
	
	
	//	Read current register value, update bits, write new value
	if (!ctrno)
	{
		// mask off bit 5
		input = _inp(BaseAddress + 4) & 0xDF;
		input += (clkfrq << 5);
		_outp(BaseAddress + 4, input);
	}
	
	if (ctrno == 1)
	{
	  // mask off bit 7,6
		input = _inp(BaseAddress + 4) & 0x3F;
		input += ( (clkfrq << 6) + (clksel << 7) );
		_outp(BaseAddress + 4, input);
	}
}	


short CPrm_DAQ::CtrSetRate(short ctrno, UCHAR rate)
{
	int actualRate, err;
	int div;
	
	div = (int)( (Frequency0 * ctrno) / (rate + 0.5) );
	
	actualRate = (Frequency0 * ctrno) / div;	
	
	if (actualRate > 0x10000 && ctrno == 1)
	{
		err = -1;
		return(err);
	}
	
	if (actualRate > (2^23) )
	{
		err = -1;
		return(err);
	}
	
	// load counter
	_outp(BaseAddress + 15, (ctrno << 7) + 2);
	_outp(BaseAddress + 12, div & 0xFF);
	_outp(BaseAddress + 13, div & 0xFF00);
	
	if(!ctrno)
	{
		_outp(BaseAddress + 14, div & 0xFF0000);
	}
	
	// enable counter
	_outp(BaseAddress + 15, (ctrno << 7) + 4);
	return(actualRate);
}	


ULONG CPrm_DAQ::CtrRead(short ctrno)
{
	ULONG ctrdata;
	UCHAR d1, d2, d3;
	
	// latch data and read it
	_outp (BaseAddress + 15, (ctrno << 7) + 0x40);
	
	if(!ctrno)
	{
		d1 = _inp(BaseAddress + 12);
		d2 = ( _inp(BaseAddress + 13) ) << 8;
		d3 = ( _inp(BaseAddress + 14) ) << 16;
		ctrdata = d1 + d2 + d3;
	}
	
	if(ctrno == 1)
	{
		d1 = _inp(BaseAddress + 12);
		d2 = ( _inp(BaseAddress + 13) ) << 8;
		ctrdata = d1 + d2;
	}
	return(ctrdata);	
}	


void CPrm_DAQ::CtrClear(short ctrno)
{
	// clear counter (does not affect enable/disable status)
	_outp(BaseAddress + 15, (ctrno << 7) + 1);
}	


void CPrm_DAQ::CtrLoad(short ctrno, ULONG data)
{
	UCHAR d1, d2, d3;

	_outp (BaseAddress + 15, (ctrno << 7) + 2);
	
	if(!ctrno)
	{
		d1= (UCHAR)(data & 0xFF);
		d2 = (UCHAR)( (data & 0xFF00) >> 8 );
		d3 = (UCHAR)( (data & 0xFF0000) >> 16 );
		_outp (BaseAddress + 12, d1);
		_outp (BaseAddress + 13, d2);
		_outp (BaseAddress + 14, d3);
	}
	
	if(ctrno == 1)
	{
		d1= (UCHAR)(data & 0xFF);
		d2 = (UCHAR)( (data & 0xFF00) >> 8 );
		_outp (BaseAddress + 12, d1);
		_outp (BaseAddress + 13, d2);
	}
}	


void CPrm_DAQ::CtrEnable(short ctrno)
{
	// start or resume counting
	_outp (BaseAddress + 15, (ctrno << 7) + 4);
}	



void CPrm_DAQ::CtrPause(short ctrno)
{
	// pause counting
	_outp (BaseAddress + 15, (ctrno << 7) + 8);
}	


void CPrm_DAQ::CtrDirectSet(UCHAR ctrconfig)
{
	// write config byte directly to counter
	_outp(BaseAddress + 15, ctrconfig);
}	

void CPrm_DAQ::CtrGateEnable(short ctrno)
{
	// enable gate for counter
	_outp (BaseAddress + 15, (ctrno << 7) + 0x10);
}	



void CPrm_DAQ::CtrGateDisable(short ctrno)
{
	// Disable gate for counter
	_outp (BaseAddress + 15, (ctrno << 7) + 0x20);
}	


int CPrm_DAQ::TimerInt(void *lpfunc, int rate)
{
	short err, input;
	int actualRate;
	
	if(Int)
	{
		err = -1;
		return(-1);
	}

	Int = 1;
		
	input = _inp(BaseAddress + 4) & 0xF0;
	_outp(BaseAddress + 4, input);
	
	
	actualRate = CtrSetRate(1, rate);
	if(actualRate == -1)
	{
		err = -1;
		return(err);
	}

	// stop timer 1 while installing interrupt routine	
	CtrPause(1);
	
	// install pointer to user function *userintfunction()
	// enable hardware interrupts on board's interrupt level (language/OS dependent)
//		lpfunc = userintfunction();

	// get current config value; strip off int and dma bits
	input = _inp(BaseAddress + 4) & 0xF0;	
	// set timer interrupt enable bit
	_outp (BaseAddress + 4, input | 4);	

	// restarts counter from the beginning
	Int = 0;
	return(actualRate);
}	


int CPrm_DAQ::DigitalInt(void *lpfunc)
{
	int err;
	USHORT input;
//	int actual_rate, div1, div2;
	
	//	If another interrupt operation is in progress then return (error)
	if(Int)
	{
		err = -1;
		return(-1);
	}

	Int = 1;

	// get current config value; strip off int and dma bits	
	input = _inp(BaseAddress + 4) & 0xF0;	
	// stop interrupts temporarily
	_outp (BaseAddress + 4, input);	


	// install pointer to user function *userintfunction()
	// enable hardware interrupts on board's interrupt level (language/OS dependent)
//		lpfunc = userintfunction();


	// get current config value; strip off int and dma bits
	input = _inp(BaseAddress + 4) & 0xF0;
	// set digital interrupt enable bit
	_outp(BaseAddress + 4, input | 2);
	
	Int = 0;
	return(0);
}	


	// formula for counter divisors for 2-counter timer circuits
	// do this if precise rate cannot be calculated from inputs
	// calculate divisors and resulting actual rate
/*
	div1 = int(sqrt(clkfrq(ctrno)/rate) = 0.5)
	div2 = int(rate/div1 + 0.5)
	actual_rate = clkfrq / div1 / div2
	
	
	//another formula
	for i = 2 to int(sqrt(clkfrq(ctrno)/rate)+0.5)
	div1 = int (clkfrq(ctrno) / i + 0.5)
	if div1 < 64K then div2 = i;	// found 2 valid divisors
	next i
	store all successful pairs, pick most accurate one
	loop cannot fail because 64K^2 > clkfrq(ctrno) so eventually both div1 and div1 < 64K

 */
